@@ -55,8 +55,8 @@
 
 extern "C" NSString* ScintillaRecPboardType;
 
-@class InnerView;
-@class MarginView;
+@class SCIContentView;
+@class SCIMarginView;
 @class ScintillaView;
 
 @class FindHighlightLayer;
@@ -77,26 +77,6 @@ extern "C" NSString* ScintillaRecPboardType;
 
 namespace Scintilla {
 
-/**
- * On the Mac, there is no WM_COMMAND or WM_NOTIFY message that can be sent
- * back to the parent. Therefore, there must be a callback handler that acts
- * like a Windows WndProc, where Scintilla can send notifications to. Use
- * ScintillaCocoa::RegisterNotifyHandler() to register such a handler.
- * Message format is:
- * <br>
- * WM_COMMAND: HIWORD (wParam) = notification code, LOWORD (wParam) = 0 (no control ID), lParam = ScintillaCocoa*
- * <br>
- * WM_NOTIFY: wParam = 0 (no control ID), lParam = ptr to SCNotification structure, with hwndFrom set to ScintillaCocoa*
- */
-typedef void(*SciNotifyFunc) (intptr_t windowid, unsigned int iMessage, uintptr_t wParam, uintptr_t lParam);
-
-/**
- * Scintilla sends these two messages to the nofity handler. Please refer
- * to the Windows API doc for details about the message format.
- */
-#define	WM_COMMAND	1001
-#define WM_NOTIFY	1002
-
 /**
  * Main scintilla class, implemented for OS X (Cocoa).
  */
@@ -105,7 +85,9 @@ class ScintillaCocoa : public ScintillaBase
 private:
   TimerTarget* timerTarget;
   NSEvent* lastMouseEvent;
-  
+
+  id<ScintillaNotificationProtocol> delegate;
+
   SciNotifyFunc	notifyProc;
   intptr_t notifyObj;
 
@@ -119,20 +101,24 @@ private:
 
   bool GetPasteboardData(NSPasteboard* board, SelectionText* selectedText);
   void SetPasteboardData(NSPasteboard* board, const SelectionText& selectedText);
-  
+
   int scrollSpeed;
   int scrollTicks;
   NSTimer* tickTimer;
   NSTimer* idleTimer;
   CFRunLoopObserverRef observer;
-	
+
   FindHighlightLayer *layerFindIndicator;
 
 protected:
   Point GetVisibleOriginInMain();
   PRectangle GetClientRectangle();
+  virtual PRectangle GetClientDrawingRectangle();
   Point ConvertPoint(NSPoint point);
-  
+  virtual void RedrawRect(PRectangle rc);
+  virtual void DiscardOverdraw();
+  virtual void Redraw();
+
   virtual void Initialise();
   virtual void Finalise();
   virtual CaseFolder *CaseFolderForEncoding();
@@ -140,15 +126,16 @@ protected:
   virtual void CancelModes();
 
 public:
-  ScintillaCocoa(InnerView* view, MarginView* viewMargin);
+  ScintillaCocoa(SCIContentView* view, SCIMarginView* viewMargin);
   virtual ~ScintillaCocoa();
 
+  void SetDelegate(id<ScintillaNotificationProtocol> delegate_);
   void RegisterNotifyCallback(intptr_t windowid, SciNotifyFunc callback);
   sptr_t WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam);
 
   ScintillaView* TopContainer();
   NSScrollView* ScrollContainer();
-  InnerView* ContentView();
+  SCIContentView* ContentView();
 
   bool SyncPaint(void* gc, PRectangle rc);
   bool Draw(NSRect rect, CGContextRef gc);
@@ -159,6 +146,7 @@ public:
   bool SetIdle(bool on);
   void SetMouseCapture(bool on);
   bool HaveMouseCapture();
+  void WillDraw(NSRect rect);
   void ScrollText(int linesToMove);
   void SetVerticalScrollPos();
   void SetHorizontalScrollPos();
@@ -188,8 +176,8 @@ public:
   virtual void ClaimSelection();
 
   NSPoint GetCaretPosition();
-  
-  static sptr_t DirectFunction(ScintillaCocoa *sciThis, unsigned int iMessage, uptr_t wParam, sptr_t lParam);
+
+  static sptr_t DirectFunction(sptr_t ptr, unsigned int iMessage, uptr_t wParam, sptr_t lParam);
 
   void TimerFired(NSTimer* timer);
   void IdleTimerFired();
@@ -218,14 +206,14 @@ public:
   void DraggingExited(id <NSDraggingInfo> info);
   bool PerformDragOperation(id <NSDraggingInfo> info);
   void DragScroll();
-  
+
   // Promote some methods needed for NSResponder actions.
   virtual void SelectAll();
   void DeleteBackward();
   virtual void Cut();
   virtual void Undo();
   virtual void Redo();
-  
+
   virtual NSMenu* CreateContextMenu(NSEvent* event);
   void HandleCommand(NSInteger command);
 
