@@ -73,6 +73,26 @@ void WEntry::SetText(const GUI::gui_char *text) {
 	return gtk_entry_set_text(GTK_ENTRY(GetID()), text);
 }
 
+void WEntry::SetValid(GtkEntry *entry, bool valid) {
+#if GTK_CHECK_VERSION(3,0,0)
+	if (valid) {
+		GdkRGBA black = { 0, 0, 0, 1};
+		gtk_widget_override_color(GTK_WIDGET(entry), (GtkStateFlags)GTK_STATE_NORMAL, &black);
+	} else {
+		GdkRGBA red = { 1.0, 0, 0, 1};
+		gtk_widget_override_color(GTK_WIDGET(entry), (GtkStateFlags)GTK_STATE_NORMAL, &red);
+	}
+#else
+	if (valid) {
+		GdkColor white = { 0, 0xFFFF, 0xFFFF, 0xFFFF};
+		gtk_widget_modify_base(GTK_WIDGET(entry), GTK_STATE_NORMAL, &white);
+	} else {
+		GdkColor red = { 0, 0xFFFF, 0x6666, 0x6666 };
+		gtk_widget_modify_base(GTK_WIDGET(entry), GTK_STATE_NORMAL, &red);
+	}
+#endif
+}
+
 void WComboBoxEntry::Create() {
 #if GTK_CHECK_VERSION(3,0,0)
 	SetID(gtk_combo_box_text_new_with_entry());
@@ -170,7 +190,7 @@ void WProgress::Create() {
 	SetID(gtk_progress_bar_new());
 }
 
-WCheckDraw::WCheckDraw() : isActive(false), pbGrey(0), over(false) {
+WCheckDraw::WCheckDraw() : isActive(false), pbGrey(0), over(false), cdfn(NULL), user(NULL) {
 #if !GTK_CHECK_VERSION(3,4,0)
 	pStyle = 0;
 #endif
@@ -205,8 +225,6 @@ static void GreyToAlpha(GdkPixbuf *ppb, GdkColor fore) {
 	}
 }
 
-const int stripIconWidth = 16;
-
 void WCheckDraw::Create(const char **xpmImage, GUI::gui_string toolTip, GtkStyle *pStyle_) {
 	isActive = false;
 	pbGrey = gdk_pixbuf_new_from_xpm_data(xpmImage);
@@ -272,6 +290,13 @@ void WCheckDraw::SetActive(bool active) {
 void WCheckDraw::Toggle() {
 	isActive = !isActive;
 	InvalidateAll();
+	if (cdfn)
+		cdfn(this, user);
+}
+
+void WCheckDraw::SetChangeFunction(ChangeFunction cdfn_, void *user_) {
+	cdfn = cdfn_;
+	user = user_;
 }
 
 gboolean WCheckDraw::Focus(GtkWidget */*widget*/, GdkEventFocus */*event*/, WCheckDraw *pcd) {
