@@ -34,17 +34,7 @@
 
 #undef _WIN32_WINNT
 #define _WIN32_WINNT  0x0500
-#ifdef _MSC_VER
-// windows.h, et al, use a lot of nameless struct/unions - can't fix it, so allow it
-#pragma warning(disable: 4201)
-#endif
 #include <windows.h>
-#ifdef _MSC_VER
-// okay, that's done, don't allow it in our code
-#pragma warning(default: 4201)
-// Turn off MS dislike of POSIX
-#pragma warning(disable: 4996)
-#endif
 #include <commctrl.h>
 
 // For chdir
@@ -209,7 +199,7 @@ FilePath FilePath::BaseName() const {
 		} else {
 			return FilePath(fileName.substr(dirEnd + 1));
 		}
-	} else if (extStart) {
+	} else if (extStart != GUI::gui_string::npos) {
 		return FilePath(fileName.substr(0, extStart));
 	} else {
 		return fileName;
@@ -298,31 +288,30 @@ static int stat(const wchar_t *path, struct _stat *buffer) {
 
 #endif
 
-static GUI::gui_char *split(GUI::gui_char*& s, GUI::gui_char c) {
-	GUI::gui_char *t = s;
-	if (s && (s = strchr(s, c)) != NULL)
-		* s++ = '\0';
-	return t;
-}
-
 FilePath FilePath::NormalizePath() const {
-	GUI::gui_char *path = new GUI::gui_char[fileName.length() + 1];
-	strcpy(path, AsInternal());
+	if (fileName.empty()) {
+		return FilePath();
+	}
+	GUI::gui_string path = fileName;
 #ifdef WIN32
 	// Convert unix path separators to Windows
-	std::replace(path, path+strlen(path), L'/', pathSepChar);
+	std::replace(path.begin(), path.end(), L'/', pathSepChar);
 #endif
-	GUI::gui_char *absPath = new GUI::gui_char[fileName.length() + 1];
+	GUI::gui_string absPathString(fileName.length() + 1, 0);
+	GUI::gui_char *absPath = &absPathString[0];
 	GUI::gui_char *cur = absPath;
 	*cur = '\0';
-	GUI::gui_char *tmp = path;
-	if (*tmp == pathSepChar) {
+	GUI::gui_char *part = &path[0];
+	if (*part == pathSepChar) {
 		*cur++ = pathSepChar;
 		*cur = '\0';
-		tmp++;
+		part++;
 	}
-	GUI::gui_char *part;
-	while ((part = split(tmp, pathSepChar)) != NULL) {
+	// Split into components and remove x/.. and .
+	while (part) {
+		GUI::gui_char *next = strchr(part, pathSepChar);
+		if (next)
+			*next++ = 0;
 		GUI::gui_char *last;
 		if (strcmp(part, GUI_TEXT(".")) == 0)
 			;
@@ -338,11 +327,9 @@ FilePath FilePath::NormalizePath() const {
 			strcpy(cur, part);
 			cur += strlen(part);
 		}
+		part = next;
 	}
-	FilePath ret(absPath);
-	delete []path;
-	delete []absPath;
-	return ret;
+	return FilePath(absPath);
 }
 
 /**
@@ -509,16 +496,16 @@ bool FilePath::IsDirectory() const {
 #endif
 	if (stat(AsInternal(), &statusFile) != -1)
 #ifdef WIN32
-		return statusFile.st_mode & _S_IFDIR;
+		return (statusFile.st_mode & _S_IFDIR) != 0;
 #else
-		return statusFile.st_mode & S_IFDIR;
+		return (statusFile.st_mode & S_IFDIR) != 0;
 #endif
 	else
 		return false;
 }
 
 #ifdef _WIN32
-void Lowercase(GUI::gui_string &s) {
+static void Lowercase(GUI::gui_string &s) {
 	int chars = ::LCMapString(LOCALE_SYSTEM_DEFAULT, LCMAP_LOWERCASE, s.c_str(), static_cast<int>(s.size())+1, NULL, 0);
 	std::vector<wchar_t> vc(chars);
 	::LCMapString(LOCALE_SYSTEM_DEFAULT, LCMAP_LOWERCASE, s.c_str(), static_cast<int>(s.size())+1, &vc[0], chars);
@@ -663,7 +650,7 @@ std::string CommandExecute(const GUI::gui_char *command, const GUI::gui_char *di
 
 	PROCESS_INFORMATION pi = {0, 0, 0, 0};
 
-	bool running = ::CreateProcessW(
+	BOOL running = ::CreateProcessW(
 			  NULL,
 			  const_cast<wchar_t *>(command),
 			  NULL, NULL,
