@@ -22,12 +22,6 @@ extern const GUI::gui_char fileWrite[];
 #endif
 
 #ifdef WIN32
-#ifdef _MSC_VER
-// Shut up level 4 warning:
-// warning C4710: function 'void whatever(...)' not inlined
-// warning C4800: forcing value to bool 'true' or 'false' (performance warning)
-#pragma warning(disable: 4710 4800)
-#endif
 #ifdef __DMC__
 #include <time.h>
 #endif
@@ -64,7 +58,6 @@ public:
 	FilePath Extension() const;
 	FilePath Directory() const;
 	void FixName();
-	void LowerCaseExtension();
 	FilePath AbsolutePath() const;
 	FilePath NormalizePath() const;
 	static FilePath GetWorkingDirectory();
