@@ -144,6 +144,20 @@ bool PropSetFile::Exists(const char *key) const {
 	}
 }
 
+std::string PropSetFile::GetString(const char *key) const {
+	const std::string sKey(key);
+	const PropSetFile *psf = this;
+	while (psf) {
+		mapss::const_iterator keyPos = psf->props.find(sKey);
+		if (keyPos != psf->props.end()) {
+			return keyPos->second;
+		}
+		// Failed here, so try in base property set
+		psf = psf->superPS;
+	}
+	return "";
+}
+
 SString PropSetFile::Get(const char *key) const {
 	const std::string sKey(key);
 	const PropSetFile *psf = this;
@@ -286,6 +300,12 @@ SString PropSetFile::GetExpanded(const char *key) const {
 	return val;
 }
 
+std::string PropSetFile::GetExpandedString(const char *key) const {
+	SString val = Get(key);
+	ExpandAllInPlace(*this, val, 100, VarChain(key));
+	return std::string(val.c_str(), val.length());
+}
+
 SString PropSetFile::Expand(const char *withVars, int maxExpands) const {
 	SString val = withVars;
 	ExpandAllInPlace(*this, val, maxExpands);
@@ -375,8 +395,10 @@ static bool GenericPropertiesFile(const FilePath &filename) {
 	return name.find("SciTE") != std::string::npos;
 }
 
-void PropSetFile::Import(FilePath filename, FilePath directoryForImports, const ImportFilter &filter, std::vector<FilePath> *imports) {
-	if (Read(filename, directoryForImports, filter, imports)) {
+void PropSetFile::Import(FilePath filename, FilePath directoryForImports, const ImportFilter &filter, std::vector<FilePath> *imports, size_t depth) {
+	if (depth > 20)	// Possibly recursive import so give up to avoid crash
+		return;
+	if (Read(filename, directoryForImports, filter, imports, depth)) {
 		if (imports && (std::find(imports->begin(),imports->end(), filename) == imports->end())) {
 			imports->push_back(filename);
 		}
@@ -384,7 +406,7 @@ void PropSetFile::Import(FilePath filename, FilePath directoryForImports, const
 }
 
 bool PropSetFile::ReadLine(const char *lineBuffer, bool ifIsTrue, FilePath directoryForImports,
-                           const ImportFilter &filter, std::vector<FilePath> *imports) {
+                           const ImportFilter &filter, std::vector<FilePath> *imports, size_t depth) {
 	//UnSlash(lineBuffer);
 	if (!IsSpaceOrTab(lineBuffer[0]))    // If clause ends with first non-indented line
 		ifIsTrue = true;
@@ -404,13 +426,13 @@ bool PropSetFile::ReadLine(const char *lineBuffer, bool ifIsTrue, FilePath direc
 					!GenericPropertiesFile(fpFile) &&
 					filter.IsValid(fpFile.BaseName().AsUTF8())) {
 					FilePath importPath(directoryForImports, fpFile);
-					Import(importPath, directoryForImports, filter, imports);
+					Import(importPath, directoryForImports, filter, imports, depth+1);
 				}
 			}
 		} else if (filter.IsValid(importName.c_str())) {
 			importName += ".properties";
 			FilePath importPath(directoryForImports, FilePath(GUI::StringFromUTF8(importName.c_str())));
-			Import(importPath, directoryForImports, filter, imports);
+			Import(importPath, directoryForImports, filter, imports, depth+1);
 		}
 	} else if (ifIsTrue && !IsCommentLine(lineBuffer)) {
 		Set(lineBuffer);
@@ -419,7 +441,7 @@ bool PropSetFile::ReadLine(const char *lineBuffer, bool ifIsTrue, FilePath direc
 }
 
 void PropSetFile::ReadFromMemory(const char *data, size_t len, FilePath directoryForImports,
-                                 const ImportFilter &filter, std::vector<FilePath> *imports) {
+                                 const ImportFilter &filter, std::vector<FilePath> *imports, size_t depth) {
 	const char *pd = data;
 	std::vector<char> lineBuffer(len+1);	// +1 for NUL
 	bool ifIsTrue = true;
@@ -432,12 +454,12 @@ void PropSetFile::ReadFromMemory(const char *data, size_t len, FilePath director
 				}
 			}
 		}
-		ifIsTrue = ReadLine(&lineBuffer[0], ifIsTrue, directoryForImports, filter, imports);
+		ifIsTrue = ReadLine(&lineBuffer[0], ifIsTrue, directoryForImports, filter, imports, depth);
 	}
 }
 
 bool PropSetFile::Read(FilePath filename, FilePath directoryForImports,
-                       const ImportFilter &filter, std::vector<FilePath> *imports) {
+                       const ImportFilter &filter, std::vector<FilePath> *imports, size_t depth) {
 	FILE *rcfile = filename.Open(fileRead);
 	if (rcfile) {
 		fseek(rcfile, 0, SEEK_END);
@@ -455,7 +477,7 @@ bool PropSetFile::Read(FilePath filename, FilePath directoryForImports,
 			data += 3;
 			lenFile -= 3;
 		}
-		ReadFromMemory(data, lenFile, directoryForImports, filter, imports);
+		ReadFromMemory(data, lenFile, directoryForImports, filter, imports, depth);
 		return true;
 	}
 	return false;
