@@ -14,17 +14,14 @@ extern const GUI::gui_char propAbbrevFileName[];
 #ifdef WIN32
 #ifdef _MSC_VER
 // Shut up level 4 warning:
-// warning C4710: function 'void whatever(...)' not inlined
 // warning C4800: forcing value to bool 'true' or 'false' (performance warning)
-#pragma warning(disable: 4710 4800)
+#pragma warning(disable: 4800)
 #endif
 #ifdef __DMC__
 #include <time.h>
 #endif
 #endif
 
-#define ELEMENTS(a) (sizeof(a) / sizeof(a[0]))
-
 inline int Minimum(int a, int b) {
 	return (a < b) ? a : b;
 }
@@ -37,11 +34,6 @@ inline long LongFromTwoShorts(short a,short b) {
 	return (a) | ((b) << 16);
 }
 
-typedef long Colour;
-inline Colour ColourRGB(unsigned int red, unsigned int green, unsigned int blue) {
-	return red | (green << 8) | (blue << 16);
-}
-
 /**
  * The order of menus on Windows - the Buffers menu may not be present
  * and there is a Help menu at the end.
@@ -101,8 +93,8 @@ public:
 	time_t fileModTime;
 	time_t fileModLastAsk;
 	time_t documentModTime;
-	enum { fmNone, fmMarked, fmModified} findMarks;
-	SString overrideExtension;	///< User has chosen to use a particular language
+	enum { fmNone, fmTemporary, fmMarked, fmModified} findMarks;
+	std::string overrideExtension;	///< User has chosen to use a particular language
 	std::vector<int> foldState;
 	std::vector<int> bookmarks;
 	FileWorker *pFileWorker;
@@ -204,19 +196,20 @@ private:
 // class to hold user defined keyboard short cuts
 class ShortcutItem {
 public:
-	SString menuKey; // the keyboard short cut
-	SString menuCommand; // the menu command to be passed to "SciTEBase::MenuCommand"
+	std::string menuKey; // the keyboard short cut
+	std::string menuCommand; // the menu command to be passed to "SciTEBase::MenuCommand"
 };
 
 class LanguageMenuItem {
 public:
-	SString menuItem;
-	SString menuKey;
-	SString extension;
+	std::string menuItem;
+	std::string menuKey;
+	std::string extension;
 };
 
 enum {
     heightTools = 24,
+    heightToolsBig = 32,
     heightTab = 24,
     heightStatus = 20,
     statusPosWidth = 256
@@ -240,34 +233,6 @@ enum IndentationStatus {
     isKeyWordStart	// Keywords that cause indentation
 };
 
-int IntFromHexDigit(int ch);
-int IntFromHexByte(const char *hexByte);
-
-class StyleDefinition {
-public:
-	SString font;
-	float sizeFractional;
-	int size;
-	SString fore;
-	SString back;
-	int weight;
-	bool italics;
-	bool eolfilled;
-	bool underlined;
-	int caseForce;
-	bool visible;
-	bool changeable;
-	enum flags { sdNone = 0, sdFont = 0x1, sdSize = 0x2, sdFore = 0x4, sdBack = 0x8,
-	        sdWeight = 0x10, sdItalics = 0x20, sdEOLFilled = 0x40, sdUnderlined = 0x80,
-	        sdCaseForce = 0x100, sdVisible = 0x200, sdChangeable = 0x400} specified;
-	StyleDefinition(const char *definition);
-	bool ParseStyleDefinition(const char *definition);
-	long ForeAsLong() const;
-	long BackAsLong() const;
-	int FractionalSize() const;
-	bool IsBold() const;
-};
-
 struct StyleAndWords {
 	int styleNumber;
 	SString words;
@@ -285,6 +250,7 @@ struct CurrentWordHighlight {
 		delayAlreadyElapsed // Delay has already elapsed, word at the caret and occurrences are (or have to be) highlighted.
 	} statesOfDelay;
 	bool isEnabled;
+	bool textHasChanged;
 	GUI::ElapsedTime elapsedTimes;
 	bool isOnlyWithSameStyle;
 
@@ -292,6 +258,7 @@ struct CurrentWordHighlight {
 		statesOfDelay = noDelay;
 		isEnabled = false;
 		isOnlyWithSameStyle = false;
+		textHasChanged = false;
 	}
 };
 
@@ -310,8 +277,8 @@ public:
 // Interface between SciTE and dialogs and strips for find and replace
 class Searcher {
 public:
-	SString findWhat;
-	SString replaceWhat;
+	std::string findWhat;
+	std::string replaceWhat;
 
 	bool wholeWord;
 	bool matchCase;
@@ -320,8 +287,10 @@ public:
 	bool wrapFind;
 	bool reverseFind;
 
+	int searchStartPosition;
 	bool replacing;
 	bool havefound;
+	bool failedfind;
 	bool findInStyle;
 	int findStyle;
 	bool closeFind;
@@ -332,17 +301,21 @@ public:
 
 	Searcher();
 
+	virtual void SetFindText(const char *sFind) = 0;
 	virtual void SetFind(const char *sFind) = 0;
 	virtual bool FindHasText() const = 0;
+	void InsertFindInMemory();
 	virtual void SetReplace(const char *sReplace) = 0;
-	virtual void MoveBack(int distance) = 0;
+	virtual void SetCaretAsStart() = 0;
+	virtual void MoveBack() = 0;
 	virtual void ScrollEditorIfNeeded() = 0;
 
-	virtual int FindNext(bool reverseDirection, bool showWarnings = true, bool allowRegExp=true) = 0;
+	virtual int FindNext(bool reverseDirection, bool showWarnings=true, bool allowRegExp=true) = 0;
 	virtual void HideMatch() = 0;
-	virtual int MarkAll() = 0;
+	enum MarkPurpose { markWithBookMarks, markIncremental };
+	virtual void MarkAll(MarkPurpose purpose=markWithBookMarks) = 0;
 	virtual int ReplaceAll(bool inSelection) = 0;
-	virtual void ReplaceOnce() = 0;
+	virtual void ReplaceOnce(bool showWarnings=true) = 0;
 	virtual void UIClosed() = 0;
 	virtual void UIHasFocus() = 0;
 	bool &FlagFromCmd(int cmd);
@@ -369,6 +342,7 @@ public:
 
 class SciTEBase : public ExtensionAPI, public Searcher, public WorkerListener {
 protected:
+	bool needIdle;
 	GUI::gui_string windowName;
 	FilePath filePath;
 	FilePath dirNameAtExecute;
@@ -383,7 +357,10 @@ protected:
 	enum { importCmdID = IDM_IMPORT };
 	ImportFilter filter;
 
-	enum { indicatorMatch = INDIC_CONTAINER, indicatorHightlightCurrentWord, indicatorSentinel };
+	enum { indicatorMatch = INDIC_CONTAINER,
+		indicatorHighlightCurrentWord,
+		indicatorSpellingMistake,
+		indicatorSentinel };
 	enum { markerBookmark = 1 };
 	ComboMemory memFiles;
 	ComboMemory memDirectory;
@@ -391,8 +368,7 @@ protected:
 	SString abbrevInsert;
 
 	enum { languageCmdID = IDM_LANGUAGE };
-	LanguageMenuItem *languageMenu;
-	int languageItems;
+	std::vector<LanguageMenuItem> languageMenu;
 
 	// an array of short cut items that are defined by the user in the properties file.
 	std::vector<ShortcutItem> shortCutItemList;
@@ -417,11 +393,10 @@ protected:
 	StyleAndWords statementEnd;
 	StyleAndWords blockStart;
 	StyleAndWords blockEnd;
-	enum { noPPC, ppcStart, ppcMiddle, ppcEnd, ppcDummy };	///< Indicate the kind of preprocessor condition line
-	char preprocessorSymbol;	///< Preprocessor symbol (in C: #)
-	StringList preprocCondStart;	///< List of preprocessor conditional start keywords (in C: if ifdef ifndef)
-	StringList preprocCondMiddle;	///< List of preprocessor conditional middle keywords (in C: else elif)
-	StringList preprocCondEnd;	///< List of preprocessor conditional end keywords (in C: endif)
+	enum PreProcKind { ppcNone, ppcStart, ppcMiddle, ppcEnd, ppcDummy };	///< Indicate the kind of preprocessor condition line
+	char preprocessorSymbol;	///< Preprocessor symbol (in C, #)
+	std::map<std::string, PreProcKind> preprocOfString; ///< Map preprocessor keywords to positions
+	/// In C, if ifdef ifndef : start, else elif : middle, endif : end.
 
 	GUI::Window wSciTE;  ///< Contains wToolBar, wTabBar, wContent, and wStatusBar
 	GUI::Window wContent;    ///< Contains wEditor and wOutput
@@ -438,7 +413,7 @@ protected:
 	bool tabMultiLine;
 	bool sbVisible;	///< @c true if status bar is visible.
 	SString sbValue;	///< Status bar text.
-	int sbNum;	///< Number of the currenly displayed status bar information.
+	int sbNum;	///< Number of the currently displayed status bar information.
 	int visHeightTools;
 	int visHeightTab;
 	int visHeightStatus;
@@ -451,7 +426,8 @@ protected:
 	bool wrap;
 	bool wrapOutput;
 	int wrapStyle;
-	bool isReadOnly;
+	int alphaIndicator;
+	bool underIndicator;
 	bool openFilesHere;
 	bool fullScreen;
 	enum { toolMax = 50 };
@@ -538,7 +514,7 @@ protected:
 
 	PropSetFile propsStatus;	// Not attached to a file but need SetInteger method.
 
-	enum { bufferMax = 100 };
+	enum { bufferMax = IDM_IMPORT - IDM_BUFFER };
 	BufferList buffers;
 
 	// Handle buffers
@@ -613,7 +589,7 @@ protected:
 	static bool Exists(const GUI::gui_char *dir, const GUI::gui_char *path, FilePath *resultPath);
 	void DiscoverEOLSetting();
 	void DiscoverIndentSetting();
-	SString DiscoverLanguage();
+	std::string DiscoverLanguage();
 	void OpenFile(long fileSize, bool suppressMessage, bool asynchronous);
 	virtual void OpenUriList(const char *) {}
 	virtual bool OpenDialog(FilePath directory, const GUI::gui_char *filter) = 0;
@@ -640,16 +616,16 @@ protected:
 	bool OpenSelected();
 	void Revert();
 	FilePath SaveName(const char *ext) const;
-	int SaveIfUnsure(bool forceQuestion = false);
-	int SaveIfUnsureAll(bool forceQuestion = false);
-	int SaveIfUnsureForBuilt();
-	bool SaveIfNotOpen(const FilePath &destFile, bool fixCase);
-	void AbandonAutomaticSave();
 	enum SaveFlags {
 	    sfNone = 0, 		// Default
 	    sfProgressVisible = 1, 	// Show in background save strip
 	    sfSynchronous = 16	// Write synchronously blocking UI
 	};
+	int SaveIfUnsure(bool forceQuestion = false, SaveFlags sf = sfProgressVisible);
+	int SaveIfUnsureAll(bool forceQuestion = false);
+	int SaveIfUnsureForBuilt();
+	bool SaveIfNotOpen(const FilePath &destFile, bool fixCase);
+	void AbandonAutomaticSave();
 	bool Save(SaveFlags sf = sfProgressVisible);
 	void SaveAs(const GUI::gui_char *file, bool fixCase);
 	virtual void SaveACopy() = 0;
@@ -659,6 +635,7 @@ protected:
 	bool PrepareBufferForSave(FilePath saveName);
 	bool SaveBuffer(FilePath saveName, SaveFlags sf);
 	virtual void SaveAsHTML() = 0;
+	void SaveToStreamRTF(std::ostream &os, int start = 0, int end = -1);
 	void SaveToRTF(FilePath saveName, int start = 0, int end = -1);
 	virtual void SaveAsRTF() = 0;
 	void SaveToPDF(FilePath saveName);
@@ -698,23 +675,25 @@ protected:
 	SString SelectionFilename();
 	void SelectionIntoProperties();
 	void SelectionIntoFind(bool stripEol = true);
-	virtual SString EncodeString(const SString &s);
+	virtual std::string EncodeString(const std::string &s);
 	virtual void Find() = 0;
 	virtual int WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, int style) = 0;
-	virtual void FindMessageBox(const SString &msg, const SString *findItem = 0) = 0;
-	int FindInTarget(const char *findWhat, int lenFind, int startPosition, int endPosition);
+	virtual void FindMessageBox(const SString &msg, const std::string *findItem = 0) = 0;
+	int FindInTarget(std::string findWhat, int startPosition, int endPosition);
+	virtual void SetFindText(const char *sFind);
 	virtual void SetFind(const char *sFind);
 	virtual bool FindHasText() const;
 	virtual void SetReplace(const char *sReplace);
-	virtual void MoveBack(int distance);
+	virtual void SetCaretAsStart();
+	virtual void MoveBack();
 	virtual void ScrollEditorIfNeeded();
-	virtual int FindNext(bool reverseDirection, bool showWarnings = true, bool allowRegExp=true);
+	virtual int FindNext(bool reverseDirection, bool showWarnings=true, bool allowRegExp=true);
 	virtual void HideMatch();
 	virtual void FindIncrement() = 0;
 	int IncrementSearchMode();
 	virtual void FindInFiles() = 0;
 	virtual void Replace() = 0;
-	void ReplaceOnce();
+	void ReplaceOnce(bool showWarnings=true);
 	int DoReplaceAll(bool inSelection); // returns number of replacements or negative value if error
 	int ReplaceAll(bool inSelection);
 	int ReplaceInBuffers();
@@ -803,7 +782,7 @@ protected:
 	virtual void ActivateWindow(const char *timestamp) = 0;
 
 	void RemoveFindMarks();
-	int MarkAll();
+	void MarkAll(MarkPurpose purpose=markWithBookMarks);
 	void BookmarkAdd(int lineno = -1);
 	void BookmarkDelete(int lineno = -1);
 	bool BookmarkPresent(int lineno = -1);
@@ -868,6 +847,7 @@ protected:
 	void SetOneStyle(GUI::ScintillaWindow &win, int style, const StyleDefinition &sd);
 	void SetStyleBlock(GUI::ScintillaWindow &win, const char *lang, int start, int last);
 	void SetStyleFor(GUI::ScintillaWindow &win, const char *language);
+	static void SetOneIndicator(GUI::ScintillaWindow &win, int indicator, const IndicatorDefinition &ind);
 	void ReloadProperties();
 
 	void CheckReload();
@@ -880,6 +860,8 @@ protected:
 	virtual void TimerStart(int mask);
 	virtual void TimerEnd(int mask);
 	void OnTimer();
+	virtual void SetIdler(bool on);
+	void OnIdle();
 
 	void SetHomeProperties();
 	void UIAvailable();
@@ -915,7 +897,7 @@ protected:
 	void Remove(Pane p, int start, int end);
 	void Insert(Pane p, int pos, const char *s);
 	void Trace(const char *s);
-	char *Property(const char *key);
+	std::string Property(const char *key);
 	void SetProperty(const char *key, const char *val);
 	void UnsetProperty(const char *key);
 	uptr_t GetInstance();
@@ -930,6 +912,8 @@ protected:
 
 	CurrentWordHighlight currentWordHighlight;
 	void HighlightCurrentWord(bool highlight);
+	MatchMarker matchMarker;
+	MatchMarker findMarker;
 public:
 
 	enum { maxParam = 4 };
