@@ -29,15 +29,7 @@
 
 #undef _WIN32_WINNT
 #define _WIN32_WINNT  0x0500
-#ifdef _MSC_VER
-// windows.h, et al, use a lot of nameless struct/unions - can't fix it, so allow it
-#pragma warning(disable: 4201)
-#endif
 #include <windows.h>
-#ifdef _MSC_VER
-// okay, that's done, don't allow it in our code
-#pragma warning(default: 4201)
-#endif
 #include <commctrl.h>
 
 // For chdir
@@ -60,6 +52,7 @@
 #include "StringList.h"
 #include "StringHelpers.h"
 #include "FilePath.h"
+#include "StyleDefinition.h"
 #include "PropSetFile.h"
 #include "StyleWriter.h"
 #include "Extender.h"
@@ -69,6 +62,7 @@
 #include "Cookie.h"
 #include "Worker.h"
 #include "FileWorker.h"
+#include "MatchMarker.h"
 #include "SciTEBase.h"
 
 const GUI::gui_char defaultSessionFileName[] = GUI_TEXT("SciTE.session");
@@ -326,8 +320,8 @@ BackgroundActivities BufferList::CountBackgroundActivities() const {
 				else
 					bg.storers++;
 				bg.fileNameLast = buffers[i].AsInternal();
-				bg.totalWork += buffers[i].pFileWorker->jobSize;
-				bg.totalProgress += buffers[i].pFileWorker->jobProgress;
+				bg.totalWork += buffers[i].pFileWorker->SizeJob();
+				bg.totalProgress += buffers[i].pFileWorker->ProgressMade();
 			}
 		}
 	}
@@ -509,7 +503,7 @@ bool SciTEBase::CanMakeRoom(bool maySaveIfDirty) {
 		return true;
 	} else if (maySaveIfDirty) {
 		// All available buffers are taken, try and close the current one
-		if (SaveIfUnsure(true) != IDCANCEL) {
+		if (SaveIfUnsure(true, static_cast<SaveFlags>(sfProgressVisible | sfSynchronous)) != IDCANCEL) {
 			// The file isn't dirty, or the user agreed to close the current one
 			return true;
 		}
@@ -526,7 +520,7 @@ void SciTEBase::ClearDocument() {
 	wEditor.Call(SCI_EMPTYUNDOBUFFER);
 	wEditor.Call(SCI_SETUNDOCOLLECTION, 1);
 	wEditor.Call(SCI_SETSAVEPOINT);
-	wEditor.Call(SCI_SETREADONLY, isReadOnly);
+	wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
 }
 
 void SciTEBase::CreateBuffers() {
@@ -583,7 +577,7 @@ void SciTEBase::LoadSessionFile(const GUI::gui_char *sessionName) {
 	}
 
 	propsSession.Clear();
-	propsSession.Read(sessionPathName, sessionPathName.Directory(), filter, NULL);
+	propsSession.Read(sessionPathName, sessionPathName.Directory(), filter, NULL, 0);
 
 	FilePath sessionFilePath = FilePath(sessionPathName).AbsolutePath();
 	// Add/update SessionPath environment variable
@@ -896,8 +890,7 @@ void SciTEBase::New() {
 	CurrentBuffer()->lifeState = Buffer::open;
 	jobQueue.isBuilding = false;
 	jobQueue.isBuilt = false;
-	isReadOnly = false;	// No sense to create an empty, read-only buffer...
-	CurrentBuffer()->isReadOnly = false;
+	CurrentBuffer()->isReadOnly = false;	// No sense to create an empty, read-only buffer...
 
 	ClearDocument();
 	DeleteFileStackMenu();
@@ -914,7 +907,6 @@ void SciTEBase::RestoreState(const Buffer &buffer, bool restoreBookmarks) {
 		codePage = SC_CP_UTF8;
 		wEditor.Call(SCI_SETCODEPAGE, codePage);
 	}
-	isReadOnly = CurrentBuffer()->isReadOnly;
 
 	// check to see whether there is saved fold state, restore
 	if (!buffer.foldState.empty()) {
@@ -1519,8 +1511,8 @@ inline bool isdigitchar(int ch) {
 	return (ch >= '0') && (ch <= '9');
 }
 
-int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
-	sourcePath[0] = '\0';
+int DecodeMessage(const char *cdoc, std::string &sourcePath, int format, int &column) {
+	sourcePath.clear();
 	column = -1; // default to not detected
 	switch (format) {
 	case SCE_ERR_PYTHON: {
@@ -1531,10 +1523,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 				const char *endPath = strchr(startPath, '\"');
 				if (endPath) {
 					ptrdiff_t length = endPath - startPath;
-					if (length > 0) {
-						strncpy(sourcePath, startPath, length);
-						sourcePath[length] = 0;
-					}
+					sourcePath.assign(startPath, length);
 					endPath++;
 					while (*endPath && !isdigitchar(*endPath)) {
 						endPath++;
@@ -1560,10 +1549,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 			for (int i = 0; cdoc[i]; i++) {
 				if (cdoc[i] == ':' && isdigitchar(cdoc[i + 1])) {
 					int sourceNumber = atoi(cdoc + i + 1) - 1;
-					if (i > 0) {
-						strncpy(sourcePath, cdoc, i);
-						sourcePath[i] = 0;
-					}
+					sourcePath.assign(cdoc, i);
 					i += 2;
 					while (isdigitchar(cdoc[i]))
 						++i;
@@ -1583,10 +1569,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 			const char *endPath = strchr(start, '(');
 			if (endPath) {
 				ptrdiff_t length = endPath - start;
-				if ((length > 0) && (length < MAX_PATH)) {
-					strncpy(sourcePath, start, length);
-					sourcePath[length] = 0;
-				}
+				sourcePath.assign(start, length);
 				endPath++;
 				return atoi(endPath) - 1;
 			}
@@ -1624,8 +1607,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 					ptrdiff_t length = space2 - space;
 
 					if (length > 0) {
-						strncpy(sourcePath, space, length);
-						sourcePath[length] = '\0';
+						sourcePath.assign(space, length);
 						return atoi(space2) - 1;
 					}
 				}
@@ -1638,8 +1620,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 			const char *line = strstr(cdoc, " line ");
 			ptrdiff_t length = line - (at + 4);
 			if (at && line && length > 0) {
-				strncpy(sourcePath, at + 4, length);
-				sourcePath[length] = 0;
+				sourcePath.assign(at + 4, length);
 				line += 6;
 				return atoi(line) - 1;
 			}
@@ -1651,8 +1632,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 			const char *line = strstr(cdoc, ":line ");
 			if (in && line && (line > in)) {
 				in += 4;
-				strncpy(sourcePath, in, line - in);
-				sourcePath[line - in] = 0;
+				sourcePath.assign(in, line - in);
 				line += 6;
 				return atoi(line) - 1;
 			}
@@ -1671,8 +1651,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 				const char *quote = strstr(fileStart, "'");
 				size_t length = quote - fileStart;
 				if (quote && length > 0) {
-					strncpy(sourcePath, fileStart, length);
-					sourcePath[length] = '\0';
+					sourcePath.assign(fileStart, length);
 				}
 				line += lenLine;
 				return atoi(line) - 1;
@@ -1696,8 +1675,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 					if (strchr("\t\n\r \"$%'*,;<>?[]^`{|}", cdoc[j])) {
 						j++;
 					}
-					strncpy(sourcePath, &cdoc[j], i - j);
-					sourcePath[i - j] = 0;
+					sourcePath.assign(&cdoc[j], i - j);
 					// Because usually the address is a searchPattern, lineNumber has to be evaluated later
 					return 0;
 				}
@@ -1715,8 +1693,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 			if (line && file && (line > file)) {
 				file += lenFile;
 				size_t length = line - file;
-				strncpy(sourcePath, file, length);
-				sourcePath[length] = '\0';
+				sourcePath.assign(file, length);
 				line += lenLine;
 				return atoi(line) - 1;
 			}
@@ -1739,8 +1716,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 						file++;
 					}
 					size_t length = strlen(file);
-					strncpy(sourcePath, file, length);
-					sourcePath[length] = '\0';
+					sourcePath.assign(file, length);
 					return atoi(line) - 1;
 				}
 			}
@@ -1764,8 +1740,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 					file++;
 					const char *endfile = strchr(file, ')');
 					size_t length = endfile - file;
-					strncpy(sourcePath, file, length);
-					sourcePath[length] = '\0';
+					sourcePath.assign(file, length);
 					line++;
 					return atoi(line) - 1;
 				}
@@ -1786,8 +1761,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 			if (line && file && (line > file)) {
 				file += lenFile;
 				size_t length = line - file;
-				strncpy(sourcePath, file, length);
-				sourcePath[length] = '\0';
+				sourcePath.assign(file, length);
 				line += lenLine;
 				return atoi(line) - 1;
 			}
@@ -1808,8 +1782,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 			if (line && file && (line > file)) {
 				file += lenFile;
 				size_t length = line - file;
-				strncpy(sourcePath, file, length);
-				sourcePath[length] = '\0';
+				sourcePath.assign(file, length);
 				line += lenLine;
 				if ((lineend > line)) {
 					return atoi(line) - 1;
@@ -1851,8 +1824,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 			const char *endPath = strchr(startPath, ':');
 			ptrdiff_t length = endPath - startPath;
 			if (length > 0) {
-				strncpy(sourcePath, startPath, length);
-				sourcePath[length] = 0;
+				sourcePath.assign(startPath, length);
 				int sourceNumber = atoi(endPath + 1) - 1;
 				return sourceNumber;
 			}
@@ -1866,8 +1838,7 @@ int DecodeMessage(const char *cdoc, char *sourcePath, int format, int &column) {
 			const char *endPath = strpbrk(startPath, "\t\r\n");
 			if (endPath) {
 				ptrdiff_t length = endPath - startPath;
-				strncpy(sourcePath, startPath, length);
-				sourcePath[length] = 0;
+				sourcePath.assign(startPath, length);
 				return 0;
 			}
 			break;
@@ -1894,7 +1865,7 @@ void SciTEBase::ShowMessages(int line) {
 		int startPosLine = wOutput.Call(SCI_POSITIONFROMLINE, line, 0);
 		int lineEnd = wOutput.Call(SCI_GETLINEENDPOSITION, line, 0);
 		SString message = GetRange(wOutput, startPosLine, lineEnd);
-		char source[MAX_PATH] = "";	
+		std::string source;
 		int column;
 		char style = acc.StyleAt(startPosLine);
 		int sourceLine = DecodeMessage(message.c_str(), source, style, column);
@@ -1902,7 +1873,7 @@ void SciTEBase::ShowMessages(int line) {
 		if (style == SCE_ERR_GCC) {
 			Chomp(message, ':');
 		}
-		GUI::gui_string sourceString = GUI::StringFromUTF8(source);
+		GUI::gui_string sourceString = GUI::StringFromUTF8(source.c_str());
 		FilePath sourcePath = FilePath(sourceString).NormalizePath();
 		if (filePath.Name().SameNameAs(sourcePath.Name())) {
 			if (style == SCE_ERR_GCC) {
@@ -1974,11 +1945,11 @@ void SciTEBase::GoMessage(int dir) {
 			wOutput.Call(SCI_MARKERADD, lookLine, 0);
 			wOutput.Call(SCI_SETSEL, startPosLine, startPosLine);
 			SString message = GetRange(wOutput, startPosLine, startPosLine + lineLength);
-			char source[MAX_PATH] = "";
+			std::string source;
 			int column;
 			long sourceLine = DecodeMessage(message.c_str(), source, style, column);
 			if (sourceLine >= 0) {
-				GUI::gui_string sourceString = GUI::StringFromUTF8(source);
+				GUI::gui_string sourceString = GUI::StringFromUTF8(source.c_str());
 				FilePath sourcePath = FilePath(sourceString).NormalizePath();
 				if (!filePath.Name().SameNameAs(sourcePath)) {
 					FilePath messagePath;
@@ -2018,7 +1989,7 @@ void SciTEBase::GoMessage(int dir) {
 							//if tag is linenumber, get line
 							sourceLine = cTag.value() - 1;
 						} else {
-							findWhat = cTag;
+							findWhat = cTag.string();
 							FindNext(false);
 							//get linenumber for marker from found position
 							sourceLine = wEditor.Call(SCI_LINEFROMPOSITION, wEditor.Call(SCI_GETCURRENTPOS));
