@@ -29,15 +29,7 @@
 
 #undef _WIN32_WINNT
 #define _WIN32_WINNT  0x0500
-#ifdef _MSC_VER
-// windows.h, et al, use a lot of nameless struct/unions - can't fix it, so allow it
-#pragma warning(disable: 4201)
-#endif
 #include <windows.h>
-#ifdef _MSC_VER
-// okay, that's done, don't allow it in our code
-#pragma warning(default: 4201)
-#endif
 #include <commctrl.h>
 
 // For chdir
@@ -56,6 +48,7 @@
 #include "StringList.h"
 #include "StringHelpers.h"
 #include "FilePath.h"
+#include "StyleDefinition.h"
 #include "PropSetFile.h"
 #include "StyleWriter.h"
 #include "Extender.h"
@@ -65,6 +58,7 @@
 #include "Cookie.h"
 #include "Worker.h"
 #include "FileWorker.h"
+#include "MatchMarker.h"
 #include "SciTEBase.h"
 #include "Utf8_16.h"
 
@@ -172,12 +166,12 @@ void SciTEBase::DiscoverEOLSetting() {
 }
 
 // Look inside the first line for a #! clue regarding the language
-SString SciTEBase::DiscoverLanguage() {
+std::string SciTEBase::DiscoverLanguage() {
 	char buf[64 * 1024];
 	int length = Minimum(LengthDocument(), sizeof(buf)-1);
 	GetRange(wEditor, 0, length, buf);
 	buf[length] = '\0';
-	SString languageOverride = "";
+	std::string languageOverride = "";
 	SString l1 = ExtractLine(buf, length);
 	if (l1.startswith("<?xml")) {
 		languageOverride = "xml";
@@ -197,9 +191,9 @@ SString SciTEBase::DiscoverLanguage() {
 		l1.substitute(' ', '\0');
 		const char *word = l1.c_str();
 		while (*word) {
-			SString propShBang("shbang.");
+			std::string propShBang("shbang.");
 			propShBang.append(word);
-			SString langShBang = props.GetExpanded(propShBang.c_str());
+			std::string langShBang = props.GetExpandedString(propShBang.c_str());
 			if (langShBang.length()) {
 				languageOverride = langShBang;
 			}
@@ -366,14 +360,14 @@ void SciTEBase::PerformDeferredTasks() {
 }
 
 void SciTEBase::CompleteOpen(OpenCompletion oc) {
-	wEditor.Call(SCI_SETREADONLY, isReadOnly);
+	wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
 
 	if (oc != ocSynchronous) {
 		ReadProperties();
 	}
 
 	if (language == "") {
-		SString languageOverride = DiscoverLanguage();
+		std::string languageOverride = DiscoverLanguage();
 		if (languageOverride.length()) {
 			CurrentBuffer()->overrideExtension = languageOverride;
 			CurrentBuffer()->lifeState = Buffer::open;
@@ -423,7 +417,7 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 
 	FilePath pathSaved = pFileStorer->path;
 	int errSaved = pFileStorer->err;
-	bool cancelledSaved = pFileStorer->cancelling;
+	bool cancelledSaved = pFileStorer->Cancelling();
 
 	// May not be found if save cancelled or buffer closed
 	if (iBuffer >= 0) {
@@ -509,6 +503,13 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 	InitialiseBuffers();
 
 	FilePath absPath = file.AbsolutePath();
+	if (!absPath.IsUntitled() && absPath.IsDirectory()) {
+		GUI::gui_string msg = LocaliseMessage("Path '^0' is a directory so can not be opened.",
+			absPath.AsInternal());
+		WindowMessageBox(wSciTE, msg, MB_ICONWARNING);
+		return false;
+	}
+
 	int index = buffers.GetDocumentByName(absPath);
 	if (index >= 0) {
 		buffers.SetVisible(index, true);
@@ -529,7 +530,7 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 		// Real file, not empty buffer
 		int maxSize = props.GetInt("max.file.size");
 		if (maxSize > 0 && size > maxSize) {
-			GUI::gui_string sSize = GUI::StringFromInteger(static_cast<int>(size));
+			GUI::gui_string sSize = GUI::StringFromInteger(size);
 			GUI::gui_string sMaxSize = GUI::StringFromInteger(maxSize);
 			GUI::gui_string msg = LocaliseMessage("File '^0' is ^1 bytes long,\n"
 			        "larger than the ^2 bytes limit set in the properties.\n"
@@ -563,7 +564,7 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 		std::string propertiesText = CommandExecute(GUI::StringFromUTF8(discoveryScript.c_str()).c_str(),
 			absPath.Directory().AsInternal());
 		if (propertiesText.size()) {
-			propsDiscovered.ReadFromMemory(propertiesText.c_str(), propertiesText.size(), absPath.Directory(), filter);
+			propsDiscovered.ReadFromMemory(propertiesText.c_str(), propertiesText.size(), absPath.Directory(), filter, NULL, 0);
 		}
 	}
 	CurrentBuffer()->props = propsDiscovered;
@@ -594,9 +595,8 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 		} else {
 			wEditor.Call(SCI_EMPTYUNDOBUFFER);
 		}
-		isReadOnly = props.GetInt("read.only");
-		CurrentBuffer()->isReadOnly = isReadOnly;
-		wEditor.Call(SCI_SETREADONLY, isReadOnly);
+		CurrentBuffer()->isReadOnly = props.GetInt("read.only");
+		wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
 	}
 	RemoveFileFromStack(filePath);
 	DeleteFileStackMenu();
@@ -718,7 +718,7 @@ bool SciTEBase::OpenSelected() {
 				if (cTag.value() > 0) {
 					wEditor.Call(SCI_GOTOLINE, cTag.value() - 1);
 				} else {
-					findWhat = cTag;
+					findWhat = cTag.string();
 					FindNext(false);
 				}
 			}
@@ -801,7 +801,7 @@ FilePath SciTEBase::SaveName(const char *ext) const {
 	return FilePath(savePath.c_str());
 }
 
-int SciTEBase::SaveIfUnsure(bool forceQuestion) {
+int SciTEBase::SaveIfUnsure(bool forceQuestion, SaveFlags sf) {
 	if (CurrentBuffer()->pFileWorker) {
 		if (CurrentBuffer()->pFileWorker->IsLoading())
 			// In semi-loaded state so refuse to save
@@ -821,12 +821,12 @@ int SciTEBase::SaveIfUnsure(bool forceQuestion) {
 			}
 			int decision = WindowMessageBox(wSciTE, msg, MB_YESNOCANCEL | MB_ICONQUESTION);
 			if (decision == IDYES) {
-				if (!Save())
+				if (!Save(sf))
 					decision = IDCANCEL;
 			}
 			return decision;
 		} else {
-			if (!Save())
+			if (!Save(sf))
 				return IDCANCEL;
 		}
 	}
@@ -1067,12 +1067,14 @@ bool SciTEBase::Save(SaveFlags sf) {
 				}
 			}
 		} else {
-			CurrentBuffer()->failedSave = true;
-			msg = LocaliseMessage(
-			            "Could not save file '^0'. Save under a different name?", filePath.AsInternal());
-			decision = WindowMessageBox(wSciTE, msg, MB_YESNO | MB_ICONWARNING);
-			if (decision == IDYES) {
-				return SaveAsDialog();
+			if (!CurrentBuffer()->failedSave) {
+				CurrentBuffer()->failedSave = true;
+				msg = LocaliseMessage(
+					"Could not save file '^0'. Save under a different name?", filePath.AsInternal());
+				decision = WindowMessageBox(wSciTE, msg, MB_YESNO | MB_ICONWARNING);
+				if (decision == IDYES) {
+					return SaveAsDialog();
+				}
 			}
 			return false;
 		}
@@ -1226,6 +1228,7 @@ public:
 		fp = fPath.Open(fileRead);
 		readAll = false;
 		exhausted = fp == NULL;
+		buffer[0] = 0;
 		pos = 0;
 		valid = 0;
 	}
@@ -1259,9 +1262,8 @@ class FileReader {
 	BufferedFile *bf;
 	int lineNum;
 	bool lastWasCR;
-	enum {bufLen = 1000};
-	char lineToCompare[bufLen+1];
-	char lineToShow[bufLen+1];
+	std::string lineToCompare;
+	std::string lineToShow;
 	bool caseSensitive;
 	// Private so FileReader objects can not be copied
 	FileReader(const FileReader &);
@@ -1277,39 +1279,38 @@ public:
 		delete bf;
 		bf = NULL;
 	}
-	char *Next() {
+	const char *Next() {
 		if (bf->Exhausted()) {
 			return NULL;
 		}
-		int i = 0;
+		lineToShow.clear();
 		while (!bf->Exhausted()) {
 			int ch = bf->NextByte();
-			if (i == 0 && lastWasCR && ch == '\n') {
+			if (lastWasCR && ch == '\n' && lineToShow.empty()) {
 				lastWasCR = false;
 			} else if (ch == '\r' || ch == '\n') {
 				lastWasCR = ch == '\r';
 				break;
-			} else if (i < bufLen) {
-				lineToShow[i++] = static_cast<char>(ch);
+			} else {
+				lineToShow.push_back(static_cast<char>(ch));
 			}
 		}
-		lineToShow[i] = '\0';
 		lineNum++;
-		strcpy(lineToCompare, lineToShow);
+		lineToCompare = lineToShow;
 		if (!caseSensitive) {
-			for (int j = 0; j < i; j++) {
+			for (unsigned int j = 0; j < lineToCompare.length(); j++) {
 				if (lineToCompare[j] >= 'A' && lineToCompare[j] <= 'Z') {
 					lineToCompare[j] = static_cast<char>(lineToCompare[j] - 'A' + 'a');
 				}
 			}
 		}
-		return lineToCompare;
+		return lineToCompare.c_str();
 	}
 	int LineNumber() const {
 		return lineNum;
 	}
 	const char *Original() const {
-		return lineToShow;
+		return lineToShow.c_str();
 	}
 	bool BufferContainsNull() {
 		return bf->BufferContainsNull();
@@ -1344,11 +1345,11 @@ void SciTEBase::GrepRecursive(GrepFlags gf, FilePath baseDir, const char *search
 			//OutputAppendStringSynchronised("\n");
 			FileReader fr(fPath, gf & grepMatchCase);
 			if ((gf & grepBinary) || !fr.BufferContainsNull()) {
-				while (char *line = fr.Next()) {
-					char *match = strstr(line, searchString);
+				while (const char *line = fr.Next()) {
+					const char *match = strstr(line, searchString);
 					if (match) {
 						if (gf & grepWholeWord) {
-							char *lineEnd = line + strlen(line);
+							const char *lineEnd = line + strlen(line);
 							while (match) {
 								if (((match == line) || !IsWordCharacter(match[-1])) &&
 								        ((match + searchLength == (lineEnd)) || !IsWordCharacter(match[searchLength]))) {
@@ -1389,7 +1390,7 @@ void SciTEBase::GrepRecursive(GrepFlags gf, FilePath baseDir, const char *search
 void SciTEBase::InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *fileTypes, const char *search, sptr_t &originalEnd) {
 	GUI::ElapsedTime commandTime;
 	if (!(gf & grepStdOut)) {
-		SString os;
+		std::string os;
 		os.append(">Internal search for \"");
 		os.append(search);
 		os.append("\" in \"");
@@ -1399,9 +1400,9 @@ void SciTEBase::InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const
 		MakeOutputVisible();
 		originalEnd += os.length();
 	}
-	SString searchString(search);
+	std::string searchString(search);
 	if (!(gf & grepMatchCase)) {
-		searchString.lowercase();
+		LowerCaseAZ(searchString);
 	}
 	GrepRecursive(gf, FilePath(directory), searchString.c_str(), fileTypes);
 	if (!(gf & grepStdOut)) {
