@@ -39,15 +39,7 @@ const GUI::gui_char menuAccessIndicator[] = GUI_TEXT("_");
 
 #undef _WIN32_WINNT
 #define _WIN32_WINNT  0x0500
-#ifdef _MSC_VER
-// windows.h, et al, use a lot of nameless struct/unions - can't fix it, so allow it
-#pragma warning(disable: 4201)
-#endif
 #include <windows.h>
-#ifdef _MSC_VER
-// okay, that's done, don't allow it in our code
-#pragma warning(default: 4201)
-#endif
 #include <commctrl.h>
 
 const GUI::gui_char menuAccessIndicator[] = GUI_TEXT("&");
@@ -58,6 +50,7 @@ const GUI::gui_char menuAccessIndicator[] = GUI_TEXT("&");
 #include "StringList.h"
 #include "StringHelpers.h"
 #include "FilePath.h"
+#include "StyleDefinition.h"
 #include "PropSetFile.h"
 #include "StyleWriter.h"
 #include "Extender.h"
@@ -67,6 +60,7 @@ const GUI::gui_char menuAccessIndicator[] = GUI_TEXT("&");
 #include "JobQueue.h"
 #include "Cookie.h"
 #include "Worker.h"
+#include "MatchMarker.h"
 #include "SciTEBase.h"
 
 void SciTEBase::SetImportMenu() {
@@ -98,7 +92,7 @@ void SciTEBase::SetLanguageMenu() {
 	for (int i = 0; i < 100; i++) {
 		DestroyMenuItem(menuLanguage, languageCmdID + i);
 	}
-	for (int item = 0; item < languageItems; item++) {
+	for (unsigned int item = 0; item < languageMenu.size(); item++) {
 		int itemID = languageCmdID + item;
 		GUI::gui_string entry = localiser.Text(languageMenu[item].menuItem.c_str());
 		if (languageMenu[item].menuKey.length()) {
@@ -158,11 +152,11 @@ void SciTEBase::ReadGlobalPropFile() {
 
 		propsBase.Clear();
 		FilePath propfileBase = GetDefaultPropertiesFileName();
-		propsBase.Read(propfileBase, propfileBase.Directory(), filter, &importFiles);
+		propsBase.Read(propfileBase, propfileBase.Directory(), filter, &importFiles, 0);
 
 		propsUser.Clear();
 		FilePath propfileUser = GetUserPropertiesFileName();
-		propsUser.Read(propfileUser, propfileUser.Directory(), filter, &importFiles);
+		propsUser.Read(propfileUser, propfileUser.Directory(), filter, &importFiles, 0);
 	}
 
 	if (!localiser.read) {
@@ -172,7 +166,7 @@ void SciTEBase::ReadGlobalPropFile() {
 
 void SciTEBase::ReadAbbrevPropFile() {
 	propsAbbrev.Clear();
-	propsAbbrev.Read(pathAbbreviations, pathAbbreviations.Directory(), filter, &importFiles);
+	propsAbbrev.Read(pathAbbreviations, pathAbbreviations.Directory(), filter, &importFiles, 0);
 }
 
 /**
@@ -188,7 +182,7 @@ void SciTEBase::ReadDirectoryPropFile() {
 		FilePath propfile = GetDirectoryPropertiesFileName();
 		props.Set("SciteDirectoryHome", propfile.Directory().AsUTF8().c_str());
 
-		propsDirectory.Read(propfile, propfile.Directory(), filter);
+		propsDirectory.Read(propfile, propfile.Directory(), filter, NULL, 0);
 	}
 }
 
@@ -203,41 +197,14 @@ void SciTEBase::ReadLocalPropFile() {
 	FilePath propfile = GetLocalPropertiesFileName();
 
 	propsLocal.Clear();
-	propsLocal.Read(propfile, propfile.Directory(), filter);
+	propsLocal.Read(propfile, propfile.Directory(), filter, NULL, 0);
 
 	props.Set("Chrome", "#C0C0C0");
 	props.Set("ChromeHighlight", "#FFFFFF");
 }
 
-int IntFromHexDigit(int ch) {
-	if ((ch >= '0') && (ch <= '9')) {
-		return ch - '0';
-	} else if (ch >= 'A' && ch <= 'F') {
-		return ch - 'A' + 10;
-	} else if (ch >= 'a' && ch <= 'f') {
-		return ch - 'a' + 10;
-	} else {
-		return 0;
-	}
-}
-
-int IntFromHexByte(const char *hexByte) {
-	return IntFromHexDigit(hexByte[0]) * 16 + IntFromHexDigit(hexByte[1]);
-}
-
-static Colour ColourFromString(const SString &s) {
-	if (s.length()) {
-		int r = IntFromHexByte(s.c_str() + 1);
-		int g = IntFromHexByte(s.c_str() + 3);
-		int b = IntFromHexByte(s.c_str() + 5);
-		return ColourRGB(r, g, b);
-	} else {
-		return 0;
-	}
-}
-
 Colour ColourOfProperty(PropSetFile &props, const char *key, Colour colourDefault) {
-	SString colour = props.GetExpanded(key);
+	std::string colour = props.GetExpandedString(key);
 	if (colour.length()) {
 		return ColourFromString(colour);
 	}
@@ -274,139 +241,6 @@ const char *SciTEBase::GetNextPropItem(
 	return pNext;
 }
 
-StyleDefinition::StyleDefinition(const char *definition) :
-		sizeFractional(10.0), size(10), fore("#000000"), back("#FFFFFF"),
-		weight(SC_WEIGHT_NORMAL), italics(false), eolfilled(false), underlined(false),
-		caseForce(SC_CASE_MIXED),
-		visible(true), changeable(true),
-		specified(sdNone) {
-	ParseStyleDefinition(definition);
-}
-
-bool StyleDefinition::ParseStyleDefinition(const char *definition) {
-	if (definition == NULL || *definition == '\0') {
-		return false;
-	}
-	char *val = StringDup(definition);
-	char *opt = val;
-	while (opt) {
-		// Find attribute separator
-		char *cpComma = strchr(opt, ',');
-		if (cpComma) {
-			// If found, we terminate the current attribute (opt) string
-			*cpComma = '\0';
-		}
-		// Find attribute name/value separator
-		char *colon = strchr(opt, ':');
-		if (colon) {
-			// If found, we terminate the current attribute name and point on the value
-			*colon++ = '\0';
-		}
-		if (0 == strcmp(opt, "italics")) {
-			specified = static_cast<flags>(specified | sdItalics);
-			italics = true;
-		}
-		if (0 == strcmp(opt, "notitalics")) {
-			specified = static_cast<flags>(specified | sdItalics);
-			italics = false;
-		}
-		if (0 == strcmp(opt, "bold")) {
-			specified = static_cast<flags>(specified | sdWeight);
-			weight = SC_WEIGHT_BOLD;
-		}
-		if (0 == strcmp(opt, "notbold")) {
-			specified = static_cast<flags>(specified | sdWeight);
-			weight = SC_WEIGHT_NORMAL;
-		}
-		if ((0 == strcmp(opt, "weight")) && colon) {
-			specified = static_cast<flags>(specified | sdWeight);
-			weight = atoi(colon);
-		}
-		if (0 == strcmp(opt, "font")) {
-			specified = static_cast<flags>(specified | sdFont);
-			font = colon;
-			font.substitute('|', ',');
-		}
-		if (0 == strcmp(opt, "fore")) {
-			specified = static_cast<flags>(specified | sdFore);
-			fore = colon;
-		}
-		if (0 == strcmp(opt, "back")) {
-			specified = static_cast<flags>(specified | sdBack);
-			back = colon;
-		}
-		if ((0 == strcmp(opt, "size")) && colon) {
-			specified = static_cast<flags>(specified | sdSize);
-			sizeFractional = static_cast<float>(atof(colon));
-			size = static_cast<int>(sizeFractional);
-		}
-		if (0 == strcmp(opt, "eolfilled")) {
-			specified = static_cast<flags>(specified | sdEOLFilled);
-			eolfilled = true;
-		}
-		if (0 == strcmp(opt, "noteolfilled")) {
-			specified = static_cast<flags>(specified | sdEOLFilled);
-			eolfilled = false;
-		}
-		if (0 == strcmp(opt, "underlined")) {
-			specified = static_cast<flags>(specified | sdUnderlined);
-			underlined = true;
-		}
-		if (0 == strcmp(opt, "notunderlined")) {
-			specified = static_cast<flags>(specified | sdUnderlined);
-			underlined = false;
-		}
-		if (0 == strcmp(opt, "case")) {
-			specified = static_cast<flags>(specified | sdCaseForce);
-			caseForce = SC_CASE_MIXED;
-			if (colon) {
-				if (*colon == 'u')
-					caseForce = SC_CASE_UPPER;
-				else if (*colon == 'l')
-					caseForce = SC_CASE_LOWER;
-			}
-		}
-		if (0 == strcmp(opt, "visible")) {
-			specified = static_cast<flags>(specified | sdVisible);
-			visible = true;
-		}
-		if (0 == strcmp(opt, "notvisible")) {
-			specified = static_cast<flags>(specified | sdVisible);
-			visible = false;
-		}
-		if (0 == strcmp(opt, "changeable")) {
-			specified = static_cast<flags>(specified | sdChangeable);
-			changeable = true;
-		}
-		if (0 == strcmp(opt, "notchangeable")) {
-			specified = static_cast<flags>(specified | sdChangeable);
-			changeable = false;
-		}
-		if (cpComma)
-			opt = cpComma + 1;
-		else
-			opt = 0;
-	}
-	delete []val;
-	return true;
-}
-
-long StyleDefinition::ForeAsLong() const {
-	return ColourFromString(fore);
-}
-
-long StyleDefinition::BackAsLong() const {
-	return ColourFromString(back);
-}
-
-int StyleDefinition::FractionalSize() const {
-	return static_cast<int>(sizeFractional * SC_FONT_SIZE_MULTIPLIER);
-}
-
-bool StyleDefinition::IsBold() const {
-	return weight > SC_WEIGHT_NORMAL;
-}
-
 void SciTEBase::SetOneStyle(GUI::ScintillaWindow &win, int style, const StyleDefinition &sd) {
 	if (sd.specified & StyleDefinition::sdItalics)
 		win.Send(SCI_STYLESETITALIC, style, sd.italics ? 1 : 0);
@@ -454,9 +288,17 @@ void SciTEBase::SetStyleFor(GUI::ScintillaWindow &win, const char *lang) {
 	SetStyleBlock(win, lang, 0, maxStyle);
 }
 
+void SciTEBase::SetOneIndicator(GUI::ScintillaWindow &win, int indicator, const IndicatorDefinition &ind) {
+	win.Call(SCI_INDICSETSTYLE, indicator, ind.style);
+	win.Call(SCI_INDICSETFORE, indicator, ind.colour);
+	win.Call(SCI_INDICSETALPHA, indicator, ind.fillAlpha);
+	win.Call(SCI_INDICSETOUTLINEALPHA, indicator, ind.outlineAlpha);
+	win.Call(SCI_INDICSETUNDER, indicator, ind.under);
+}
+
 SString SciTEBase::ExtensionFileName() {
 	if (CurrentBuffer()->overrideExtension.length()) {
-		return CurrentBuffer()->overrideExtension;
+		return CurrentBuffer()->overrideExtension.c_str();
 	} else {
 		FilePath name = FileNameExt();
 		if (name.IsSet()) {
@@ -531,9 +373,11 @@ void SciTEBase::ReadAPI(const SString &fileNameForExtension) {
 					if (fp) {
 						fseek(fp, 0, SEEK_END);
 						long len = ftell(fp);
-						fseek(fp, 0, SEEK_SET);
-						size_t readBytes = fread(buffer + tlen, 1, len, fp);
-						tlen += readBytes;
+						if (len > 0) {
+							fseek(fp, 0, SEEK_SET);
+							size_t readBytes = fread(buffer + tlen, 1, len, fp);
+							tlen += readBytes;
+						}
 						fclose(fp);
 					}
 					apiFileName += strlen(apiFileName) + 1;
@@ -609,6 +453,12 @@ static const char *propertiesToForward[] = {
 	"fold.preprocessor",
 	"fold.quotes.nimrod",
 	"fold.quotes.python",
+	"fold.rust.comment.explicit",
+	"fold.rust.comment.multiline",
+	"fold.rust.explicit.anywhere",
+	"fold.rust.explicit.end",
+	"fold.rust.explicit.start",
+	"fold.rust.syntax.based",
 	"fold.sql.at.else",
 	"fold.sql.only.begin",
 	"fold.verilog.flags",
@@ -616,6 +466,8 @@ static const char *propertiesToForward[] = {
 	"lexer.asm.comment.delimiter",
 	"lexer.caml.magic",
 	"lexer.cpp.allow.dollars",
+	"lexer.cpp.backquoted.strings",
+	"lexer.cpp.escape.sequence",
 	"lexer.cpp.hashquoted.strings",
 	"lexer.cpp.track.preprocessor",
 	"lexer.cpp.triplequoted.strings",
@@ -642,6 +494,7 @@ static const char *propertiesToForward[] = {
 	"lexer.python.strings.b",
 	"lexer.python.strings.over.newline",
 	"lexer.python.strings.u",
+	"lexer.rust.fold.at.else",
 	"lexer.sql.allow.dotted.word",
 	"lexer.sql.backticks.identifier",
 	"lexer.sql.numbersign.comment",
@@ -751,7 +604,7 @@ static const char *bookmarkBluegem[] = {
 };
 
 SString SciTEBase::GetFileNameProperty(const char *name) {
-	SString namePlusDot = name;
+	std::string namePlusDot = name;
 	namePlusDot.append(".");
 	SString valueForFileName = props.GetNewExpand(namePlusDot.c_str(),
 	        ExtensionFileName().c_str());
@@ -882,7 +735,7 @@ void SciTEBase::ReadProperties() {
 	wEditor.Call(SCI_SETCARETWIDTH, props.GetInt("caret.width", 1));
 	wOutput.Call(SCI_SETCARETWIDTH, props.GetInt("caret.width", 1));
 
-	SString caretLineBack = props.Get("caret.line.back");
+	std::string caretLineBack = props.GetString("caret.line.back");
 	if (caretLineBack.length()) {
 		wEditor.Call(SCI_SETCARETLINEVISIBLE, 1);
 		wEditor.Call(SCI_SETCARETLINEBACK, ColourFromString(caretLineBack));
@@ -892,22 +745,10 @@ void SciTEBase::ReadProperties() {
 	wEditor.Call(SCI_SETCARETLINEBACKALPHA,
 		props.GetInt("caret.line.back.alpha", SC_ALPHA_NOALPHA));
 
-	int alphaIndicator = props.GetInt("indicators.alpha", 30);
+	alphaIndicator = props.GetInt("indicators.alpha", 30);
 	if (alphaIndicator < 0 || 255 < alphaIndicator) // If invalid value,
 		alphaIndicator = 30; //then set default value.
-	bool underIndicator = props.GetInt("indicators.under", 0) == 1;
-	for (int index = INDIC_CONTAINER; index < indicatorSentinel; ++index) {
-		wEditor.Call(SCI_INDICSETALPHA, index, alphaIndicator);
-		wOutput.Call(SCI_INDICSETALPHA, index, alphaIndicator);
-		wEditor.Call(SCI_INDICSETUNDER, index, underIndicator);
-		wOutput.Call(SCI_INDICSETUNDER, index, underIndicator);
-	}
-
-	SString findMark = props.Get("find.mark");
-	if (findMark.length()) {
-		wEditor.Call(SCI_INDICSETSTYLE, indicatorMatch, INDIC_ROUNDBOX);
-		wEditor.Call(SCI_INDICSETFORE, indicatorMatch, ColourFromString(findMark));
-	}
+	underIndicator = props.GetInt("indicators.under", 0) == 1;
 
 	closeFind = props.GetInt("find.close.on.find", 1);
 
@@ -948,13 +789,13 @@ void SciTEBase::ReadProperties() {
 	wEditor.Call(SCI_SETEDGECOLOUR,
 	           ColourOfProperty(props, "edge.colour", ColourRGB(0xff, 0xda, 0xda)));
 
-	SString selFore = props.Get("selection.fore");
+	std::string selFore = props.GetString("selection.fore");
 	if (selFore.length()) {
 		CallChildren(SCI_SETSELFORE, 1, ColourFromString(selFore));
 	} else {
 		CallChildren(SCI_SETSELFORE, 0, 0);
 	}
-	SString selBack = props.Get("selection.back");
+	std::string selBack = props.GetString("selection.back");
 	if (selBack.length()) {
 		CallChildren(SCI_SETSELBACK, 1, ColourFromString(selBack));
 	} else {
@@ -966,37 +807,37 @@ void SciTEBase::ReadProperties() {
 	int selectionAlpha = props.GetInt("selection.alpha", SC_ALPHA_NOALPHA);
 	CallChildren(SCI_SETSELALPHA, selectionAlpha);
 
-	SString selAdditionalFore = props.Get("selection.additional.fore");
+	std::string selAdditionalFore = props.GetString("selection.additional.fore");
 	if (selAdditionalFore.length()) {
 		CallChildren(SCI_SETADDITIONALSELFORE, ColourFromString(selAdditionalFore));
 	}
-	SString selAdditionalBack = props.Get("selection.additional.back");
+	std::string selAdditionalBack = props.GetString("selection.additional.back");
 	if (selAdditionalBack.length()) {
 		CallChildren(SCI_SETADDITIONALSELBACK, ColourFromString(selAdditionalBack));
 	}
 	int selectionAdditionalAlpha = (selectionAlpha == SC_ALPHA_NOALPHA) ? SC_ALPHA_NOALPHA : selectionAlpha / 2;
 	CallChildren(SCI_SETADDITIONALSELALPHA, props.GetInt("selection.additional.alpha", selectionAdditionalAlpha));
 
-	SString foldColour = props.Get("fold.margin.colour");
+	std::string foldColour = props.GetString("fold.margin.colour");
 	if (foldColour.length()) {
 		CallChildren(SCI_SETFOLDMARGINCOLOUR, 1, ColourFromString(foldColour));
 	} else {
 		CallChildren(SCI_SETFOLDMARGINCOLOUR, 0, 0);
 	}
-	SString foldHiliteColour = props.Get("fold.margin.highlight.colour");
+	std::string foldHiliteColour = props.GetString("fold.margin.highlight.colour");
 	if (foldHiliteColour.length()) {
 		CallChildren(SCI_SETFOLDMARGINHICOLOUR, 1, ColourFromString(foldHiliteColour));
 	} else {
 		CallChildren(SCI_SETFOLDMARGINHICOLOUR, 0, 0);
 	}
 
-	SString whitespaceFore = props.Get("whitespace.fore");
+	std::string whitespaceFore = props.GetString("whitespace.fore");
 	if (whitespaceFore.length()) {
 		CallChildren(SCI_SETWHITESPACEFORE, 1, ColourFromString(whitespaceFore));
 	} else {
 		CallChildren(SCI_SETWHITESPACEFORE, 0, 0);
 	}
-	SString whitespaceBack = props.Get("whitespace.back");
+	std::string whitespaceBack = props.GetString("whitespace.back");
 	if (whitespaceBack.length()) {
 		CallChildren(SCI_SETWHITESPACEBACK, 1, ColourFromString(whitespaceBack));
 	} else {
@@ -1072,6 +913,9 @@ void SciTEBase::ReadProperties() {
 	wOutput.Call(SCI_SETMARGINLEFT, 0, blankMarginLeft);
 	wOutput.Call(SCI_SETMARGINRIGHT, 0, blankMarginRight);
 
+	marginWidth = props.GetInt("margin.width");
+	if (marginWidth == 0)
+		marginWidth = marginWidthDefault;
 	wEditor.Call(SCI_SETMARGINWIDTHN, 1, margin ? marginWidth : 0);
 
 	SString lineMarginProp = props.Get("line.margin.width");
@@ -1130,18 +974,25 @@ void SciTEBase::ReadProperties() {
 	blockStart = GetStyleAndWords("block.start.");
 	blockEnd = GetStyleAndWords("block.end.");
 
+	struct {
+		const char *propName;
+		PreProcKind ppc;
+	} propToPPC[] = {
+		{"preprocessor.start.", ppcStart},
+		{"preprocessor.middle.", ppcMiddle},
+		{"preprocessor.end.", ppcEnd},
+	};
 	SString list;
 	list = props.GetNewExpand("preprocessor.symbol.", fileNameForExtension.c_str());
 	preprocessorSymbol = list[0];
-	list = props.GetNewExpand("preprocessor.start.", fileNameForExtension.c_str());
-	preprocCondStart.Clear();
-	preprocCondStart.Set(list.c_str());
-	list = props.GetNewExpand("preprocessor.middle.", fileNameForExtension.c_str());
-	preprocCondMiddle.Clear();
-	preprocCondMiddle.Set(list.c_str());
-	list = props.GetNewExpand("preprocessor.end.", fileNameForExtension.c_str());
-	preprocCondEnd.Clear();
-	preprocCondEnd.Set(list.c_str());
+	preprocOfString.clear();
+	for (size_t iPreproc = 0; iPreproc < ELEMENTS(propToPPC); iPreproc++) {
+		list = props.GetNewExpand(propToPPC[iPreproc].propName, fileNameForExtension.c_str());
+		std::vector<std::string> words = StringSplit(std::string(list.c_str()), ' ');
+		for (std::vector<std::string>::iterator it = words.begin(); it != words.end(); ++it) {
+			preprocOfString[*it] = propToPPC[iPreproc].ppc;
+		}
+	}
 
 	memFiles.AppendList(props.GetNewExpand("find.files").c_str());
 
@@ -1214,6 +1065,9 @@ void SciTEBase::ReadProperties() {
 	// Create a margin column for the folding symbols
 	wEditor.Call(SCI_SETMARGINTYPEN, 2, SC_MARGIN_SYMBOL);
 
+	foldMarginWidth = props.GetInt("fold.margin.width");
+	if (foldMarginWidth == 0)
+		foldMarginWidth = foldMarginWidthDefault;
 	wEditor.Call(SCI_SETMARGINWIDTHN, 2, foldMargin ? foldMarginWidth : 0);
 
 	wEditor.Call(SCI_SETMARGINMASKN, 2, SC_MASK_FOLDERS);
@@ -1222,7 +1076,7 @@ void SciTEBase::ReadProperties() {
 	// Enable/disable highlight for current folding bloc (smallest one that contains the caret)
 	int isHighlightEnabled = props.GetInt("fold.highlight", 0);
 	// Define the colour of highlight
-	SString foldBlockHighlight = props.Get("fold.highlight.colour");
+	std::string foldBlockHighlight = props.GetString("fold.highlight.colour");
 	if (foldBlockHighlight.length() == 0) {
 		//Set default colour for highlight
 		foldBlockHighlight = "#FF0000";
@@ -1306,9 +1160,9 @@ void SciTEBase::ReadProperties() {
 	}
 
 	wEditor.Call(SCI_MARKERSETFORE, markerBookmark,
-		ColourOfProperty(props, "bookmark.fore", ColourRGB(0, 0, 0x7f)));
+		ColourOfProperty(props, "bookmark.fore", ColourRGB(0xbe, 0, 0)));
 	wEditor.Call(SCI_MARKERSETBACK, markerBookmark,
-		ColourOfProperty(props, "bookmark.back", ColourRGB(0x80, 0xff, 0xff)));
+		ColourOfProperty(props, "bookmark.back", ColourRGB(0xe2, 0x40, 0x40)));
 	wEditor.Call(SCI_MARKERSETALPHA, markerBookmark,
 		props.GetInt("bookmark.alpha", SC_ALPHA_NOALPHA));
 	SString bookMarkXPM = props.Get("bookmark.pixmap");
@@ -1316,7 +1170,7 @@ void SciTEBase::ReadProperties() {
 		wEditor.CallString(SCI_MARKERDEFINEPIXMAP, markerBookmark,
 			bookMarkXPM.c_str());
 	} else if (props.Get("bookmark.fore").length()) {
-		wEditor.Call(SCI_MARKERDEFINE, markerBookmark, SC_MARK_CIRCLE);
+		wEditor.Call(SCI_MARKERDEFINE, markerBookmark, SC_MARK_BOOKMARK);
 	} else {
 		// No bookmark.fore setting so display default pixmap.
 		wEditor.CallString(SCI_MARKERDEFINEPIXMAP, markerBookmark,
@@ -1336,25 +1190,29 @@ void SciTEBase::ReadProperties() {
 	wEditor.Call(SCI_SETCARETSTICKY, props.GetInt("caret.sticky", 0));
 
 	// Clear all previous indicators.
-	wEditor.Call(SCI_SETINDICATORCURRENT, indicatorHightlightCurrentWord);
+	wEditor.Call(SCI_SETINDICATORCURRENT, indicatorHighlightCurrentWord);
 	wEditor.Call(SCI_INDICATORCLEARRANGE, 0, wEditor.Call(SCI_GETLENGTH));
-	wOutput.Call(SCI_SETINDICATORCURRENT, indicatorHightlightCurrentWord);
+	wOutput.Call(SCI_SETINDICATORCURRENT, indicatorHighlightCurrentWord);
 	wOutput.Call(SCI_INDICATORCLEARRANGE, 0, wOutput.Call(SCI_GETLENGTH));
 	currentWordHighlight.statesOfDelay = currentWordHighlight.noDelay;
 
 	currentWordHighlight.isEnabled = props.GetInt("highlight.current.word", 0) == 1;
 	if (currentWordHighlight.isEnabled) {
-		SString highlightCurrentWordColourString = props.Get("highlight.current.word.colour");
-		if (highlightCurrentWordColourString.length() == 0) {
-			// Set default colour for highlight.
-			highlightCurrentWordColourString = "#A0A000";
+		SString highlightCurrentWordIndicatorString = props.Get("highlight.current.word.indicator");
+		IndicatorDefinition highlightCurrentWordIndicator(highlightCurrentWordIndicatorString.c_str());
+		if (highlightCurrentWordIndicatorString.length() == 0) {
+			highlightCurrentWordIndicator.style = INDIC_ROUNDBOX;
+			std::string highlightCurrentWordColourString = props.GetString("highlight.current.word.colour");
+			if (highlightCurrentWordColourString.length() == 0) {
+				// Set default colour for highlight.
+				highlightCurrentWordColourString = "#A0A000";
+			}
+			highlightCurrentWordIndicator.colour = ColourFromString(highlightCurrentWordColourString);
+			highlightCurrentWordIndicator.fillAlpha = alphaIndicator;
+			highlightCurrentWordIndicator.under = underIndicator;
 		}
-		Colour highlightCurrentWordColour = ColourFromString(highlightCurrentWordColourString);
-
-		wEditor.Call(SCI_INDICSETSTYLE, indicatorHightlightCurrentWord, INDIC_ROUNDBOX);
-		wEditor.Call(SCI_INDICSETFORE, indicatorHightlightCurrentWord, highlightCurrentWordColour);
-		wOutput.Call(SCI_INDICSETSTYLE, indicatorHightlightCurrentWord, INDIC_ROUNDBOX);
-		wOutput.Call(SCI_INDICSETFORE, indicatorHightlightCurrentWord, highlightCurrentWordColour);
+		SetOneIndicator(wEditor, indicatorHighlightCurrentWord, highlightCurrentWordIndicator);
+		SetOneIndicator(wOutput, indicatorHighlightCurrentWord, highlightCurrentWordIndicator);
 		currentWordHighlight.isOnlyWithSameStyle = props.GetInt("highlight.current.word.by.style", 0) == 1;
 		HighlightCurrentWord(true);
 	}
@@ -1481,17 +1339,8 @@ void SciTEBase::SetPropertiesInitial() {
 	tabVisible = props.GetInt("tabbar.visible");
 	tabMultiLine = props.GetInt("tabbar.multiline");
 	lineNumbers = props.GetInt("line.margin.visible");
-	marginWidth = 0;
-	SString margwidth = props.Get("margin.width");
-	if (margwidth.length())
-		marginWidth = margwidth.value();
-	margin = marginWidth;
-	if (marginWidth == 0)
-		marginWidth = marginWidthDefault;
-	foldMarginWidth = props.GetInt("fold.margin.width", foldMarginWidthDefault);
-	foldMargin = foldMarginWidth;
-	if (foldMarginWidth == 0)
-		foldMarginWidth = foldMarginWidthDefault;
+	margin = props.GetInt("margin.width");
+	foldMargin = props.GetInt("fold.margin.width", foldMarginWidthDefault);
 
 	matchCase = props.GetInt("find.replace.matchcase");
 	regExp = props.GetInt("find.replace.regexp");
@@ -1562,7 +1411,7 @@ void SciTEBase::ReadLocalization() {
 	}
 	FilePath propdir = GetSciteDefaultHome();
 	FilePath localePath(propdir, title);
-	localiser.Read(localePath, propdir, filter, &importFiles);
+	localiser.Read(localePath, propdir, filter, &importFiles, 0);
 	localiser.SetMissing(props.Get("translation.missing"));
 	localiser.read = true;
 }
@@ -1592,23 +1441,17 @@ void SciTEBase::ReadPropertiesInitial() {
 	wOutput.Call(SCI_SETWRAPMODE, wrapOutput ? wrapStyle : SC_WRAP_NONE);
 
 	SString menuLanguageProp = props.GetNewExpand("menu.language");
-	languageItems = 0;
-	for (unsigned int i = 0; i < menuLanguageProp.length(); i++) {
-		if (menuLanguageProp[i] == '|')
-			languageItems++;
-	}
-	languageItems /= 3;
-	languageMenu = new LanguageMenuItem[languageItems];
-
 	menuLanguageProp.substitute('|', '\0');
 	const char *sMenuLanguage = menuLanguageProp.c_str();
-	for (int item = 0; item < languageItems; item++) {
-		languageMenu[item].menuItem = sMenuLanguage;
+	while (*sMenuLanguage) {
+		LanguageMenuItem lmi;
+		lmi.menuItem = sMenuLanguage;
 		sMenuLanguage += strlen(sMenuLanguage) + 1;
-		languageMenu[item].extension = sMenuLanguage;
+		lmi.extension = sMenuLanguage;
 		sMenuLanguage += strlen(sMenuLanguage) + 1;
-		languageMenu[item].menuKey = sMenuLanguage;
+		lmi.menuKey = sMenuLanguage;
 		sMenuLanguage += strlen(sMenuLanguage) + 1;
+		languageMenu.push_back(lmi);
 	}
 	SetLanguageMenu();
 
