@@ -607,7 +607,7 @@ void SciTEWin::Print(
 			lengthDoc = lengthDocMax;
 	}
 
-	// We must substract the physical margins from the printable area
+	// We must subtract the physical margins from the printable area
 	Sci_RangeToFormat frPrint;
 	frPrint.hdc = hdc;
 	frPrint.hdcTarget = hdc;
@@ -779,7 +779,7 @@ public:
 		return s;
 	}
 
-	void SetItemTextU(int id, const SString &s) {
+	void SetItemTextU(int id, const std::string &s) {
 		SetItemText(id, GUI::StringFromUTF8(s.c_str()).c_str());
 	}
 
@@ -944,7 +944,7 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 				wFindReplace.Destroy();
 			}
 			if (ControlIDOfWParam(wParam) == IDMARKALL){
-				MarkAll();
+				MarkAll(markWithBookMarks);
 			}
 			FindNext(reverseFind ^ IsKeyDown(VK_SHIFT));
 			return TRUE;
@@ -1062,8 +1062,11 @@ void SciTEWin::FindIncrement() {
 	if (replaceStrip.visible)
 		replaceStrip.Close();
 	searchStrip.visible = !searchStrip.visible;
+	failedfind = false;
 	SizeSubWindows();
 	if (searchStrip.visible) {
+		SetCaretAsStart();
+		findWhat = "";
 		searchStrip.Focus();
 	} else {
 		WindowSetFocus(wEditor);
@@ -1094,6 +1097,7 @@ void SciTEWin::Find() {
 			replaceStrip.Close();
 		findStrip.visible = true;
 		SizeSubWindows();
+		findStrip.SetIncrementalBehaviour(props.GetInt("find.strip.incremental"));
 		findStrip.Show();
 	} else {
 		if (searchStrip.visible || replaceStrip.visible)
@@ -1173,8 +1177,8 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	case WM_INITDIALOG:
 		LocaliseDialog(hDlg);
 		FillCombos(dlg);
-		dlg.SetItemTextU(IDFINDWHAT, props.Get("find.what"));
-		dlg.SetItemTextU(IDDIRECTORY, props.Get("find.directory"));
+		dlg.SetItemTextU(IDFINDWHAT, props.GetString("find.what"));
+		dlg.SetItemTextU(IDDIRECTORY, props.GetString("find.directory"));
 		if (props.GetNewExpand("find.command") == "") {
 			// Empty means use internal that can respond to flags
 			dlg.SetCheck(IDWHOLEWORD, wholeWord);
@@ -1201,9 +1205,9 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 				WindowMessageBox(wFindInFiles, msgBuf, MB_OK | MB_ICONWARNING);
 				return FALSE;
 			}
-			findWhat = dlg.ItemTextU(IDFINDWHAT);
+			findWhat = dlg.ItemTextU(IDFINDWHAT).string();
 			props.Set("find.what", findWhat.c_str());
-			memFinds.Insert(findWhat.c_str());
+			InsertFindInMemory();
 
 			SString files = dlg.ItemTextU(IDFILES);
 			props.Set("find.files", files.c_str());
@@ -1331,6 +1335,7 @@ void SciTEWin::Replace() {
 			findStrip.Close();
 		replaceStrip.visible = true;
 		SizeSubWindows();
+		replaceStrip.SetIncrementalBehaviour(props.GetInt("replace.strip.incremental"));
 		replaceStrip.Show();
 		havefound = false;
 	} else {
@@ -1637,7 +1642,7 @@ int SciTEWin::WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, int s
 	return ret;
 }
 
-void SciTEWin::FindMessageBox(const SString &msg, const SString *findItem) {
+void SciTEWin::FindMessageBox(const SString &msg, const std::string *findItem) {
 	if (findItem == 0) {
 		GUI::gui_string msgBuf = LocaliseMessage(msg.c_str());
 		WindowMessageBox(wFindReplace.Created() ? wFindReplace : wSciTE, msgBuf, MB_OK | MB_ICONWARNING);
